/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.cocoon.pipeline.component.sax;

import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Map;

import org.apache.cocoon.pipeline.caching.CacheKey;
import org.apache.cocoon.pipeline.caching.TimestampCacheKey;
import org.apache.cocoon.pipeline.component.CachingPipelineComponent;
import org.apache.cocoon.pipeline.util.StringRepresentation;
import org.apache.cocoon.pipeline.util.URLConnectionUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class FileGenerator extends AbstractGenerator implements CachingPipelineComponent {

    private final Log logger = LogFactory.getLog(this.getClass());

    private URL source;

    public FileGenerator() {
        super();
    }

    public FileGenerator(URL source) {
        super();
        this.source = source;
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.apache.cocoon.pipeline.component.CachingPipelineComponent#constructCacheKey()
     */
    public CacheKey constructCacheKey() {
        try {
            URLConnection connection = this.source.openConnection();
            TimestampCacheKey timestampCacheKey = new TimestampCacheKey(this.source, connection.getLastModified());
            URLConnectionUtils.closeQuietly(connection);
            return timestampCacheKey;
        } catch (IOException e) {
            this.logger.error("Can't construct cache key. Error while connecting to " + this.source, e);
        }

        return null;
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.apache.cocoon.pipeline.component.Starter#execute()
     */
    public void execute() {
        if (this.source == null) {
            throw new IllegalArgumentException("FileGenerator has no source.");
        }

        try {
            URLConnectionUtils.toSax(this.source.openConnection(), this.getXMLConsumer());
        } catch (IOException e) {
            throw new RuntimeException("Can't open connection to " + this.source, e);
        }
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.apache.cocoon.pipeline.component.sax.AbstractXMLProducer#setConfiguration(java.util.Map)
     */
    @Override
    public void setConfiguration(Map<String, ? extends Object> configuration) {
        this.setSource((URL) configuration.get("source"));
    }

    public void setSource(URL source) {
        this.source = source;
    }

    @Override
    public String toString() {
        return StringRepresentation.buildString(this, "source=" + this.source);
    }
}
